#  tests for seabreeze-1.0.0-py38h6538335_0 (this is a generated file);
print('===== testing package: seabreeze-1.0.0-py38h6538335_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import os
import pytest

# workaround to undo conda-forge settng CI env var
os.environ['CI'] = 'true'
test_prefix = os.environ['PREFIX']

pytest.main(['-v', os.path.join(test_prefix, 'tests')])#  --- run_test.py (end) ---

print('===== seabreeze-1.0.0-py38h6538335_0 OK =====');
print("import: 'seabreeze'")
import seabreeze

print("import: 'seabreeze.cseabreeze'")
import seabreeze.cseabreeze

print("import: 'seabreeze.pyseabreeze'")
import seabreeze.pyseabreeze

